
--[[

This file loads the global settings file on start up and
includes an global functions to deal with loading and saving
of numbers in the different formats (euro/non-euro).

]]--

-- global function to decode (convert from strings) stored numbers
function _G.DecodeNumbers(object)
	-- parse strings into numbers where applicable
	if (type(object) == "string") then
		local number,separator,decimal = object:match("<num>(-?%d+)([%.,])(%d+)</num>");
		if (number ~= nil) then
			local decodedNumber = tonumber(number..separator..decimal);
			if (decodedNumber ~= nil) then return decodedNumber end
			return tonumber(number..(separator == "," and "." or ",")..decimal);
		else
			return object;
		end
	-- return any other data types
	elseif (type(object) ~= "table") then
		return object;
	-- decode each element inside a table and return
	else
		local newTable = {}
		for index, value in pairs(object) do
			newTable[DecodeNumbers(index)] = DecodeNumbers(value);
		end
		return newTable;
	end
end

-- load the plugin settings (will be done once on startup)
function _G.LoadSettings()
	-- 1) Load Settings file
	_G.settings = Turbine.PluginData.Load(Turbine.DataScope.Character,"CombatAnalysis");
	-- apply number formatting restoration
	_G.settings = DecodeNumbers(settings);
	
	local upToDate = true;
	
	-- if no settings file exists (or it is outdated), use the following defaults
	--   (note that forwards compatability is now assumed when finding old settings files and
	--    we retain the previous settings info where applicable - though any settings file before 
	--    version 4.0.0 will be completely overwritten)
	if type(settings) ~= "table" or settings.versionNo == nil or string.gsub(settings.versionNo,"%.","") < string.gsub(versionNo,"%.","") then
		upToDate = false;
		
		if type(settings) ~= "table" then
			_G.settings = {};
		end
		
		-- v4.0.0 settings
		if (settings.versionNo == nil or string.gsub(settings.versionNo,"%.","") < "400") then
		
			-- a list of the stat overview tabs (key = tab type, where (1 = dmgTab, 2 = takenTab, 3 = healTab, 4 = powerTab))
			settings.tabStates = {
				-- tab index in window, selected, tab window id, stats panel window id,
				--   showStats state (1->3), auto select player, show title bar, show encounters, show targets, show extra buttons,
				--   expanded stats tree headings (main,crits,avoids,fulls,partials,other,dmg types)
				[1] = {1,true,1,2,3,false,true,true,true,true,{true,true,true,true,true,true,true}},
				[2] = {2,false,1,2,3,false,true,true,true,true,{true,true,true,true,true,true,true}},
				[3] = {3,false,1,2,3,false,true,true,true,true,{true,true}},
				[4] = {4,false,1,2,3,false,true,true,true,true,{true,true}}
			};
			
			-- a list of all std stat overview windows (key = window id)
			settings.statOverviewWindowStates = {
				-- class type, xpos (nil = default), ypos (nil = default), width, height, show background, title bar width (0 or 42), auto hide tabs, minimized, unminimized width, unminimized height
				[1] = {nil,nil,270,221,true,42,false,false,270,221}
			};
			
			-- a list of all stat overview stats (details) windows (key = window id)
			settings.statOverviewStatsWindowStates = {
				-- class type, xpos (nil = default), ypos (nil = default), width, height, window id
				[2] = {nil,nil,270,221}
			};
			
			-- global settings (just the one in v4.0.0)
			settings.autoSelectNewEncounters = true;
			
		end
		
		-- v4.0.4 additional settings
		if (settings.versionNo == nil or string.gsub(settings.versionNo,"%.","") < "404") then
			-- /say chat channel selected by default
			settings.chatChannel = 1;
		end
		
		-- v4.1.0 additional settings (note that we perform a complete conversion of the old settings file format to the new more readable format)
		if (settings.versionNo == nil or string.gsub(settings.versionNo,"%.","") < "410") then
			-- 1) update tab state data
			local dmgTabData = settings.tabStates[1];
			local takenTabData = settings.tabStates[2];
			local healTabData = settings.tabStates[3];
			local powerTabData = settings.tabStates[4];
			
			settings.tabStates = {
				["dmgTab"] = {
					["indexInWindow"] = dmgTabData[1], ["selected"] = dmgTabData[2], ["windowID"] = dmgTabData[3], ["statsWindowID"] = dmgTabData[4],
					["statsWindowVisibility"] = (dmgTabData[5] == 1 and "Hide" or (dmgTabData[5] == 2 and "Hover" or "Show")), ["autoSelectPlayer"] = dmgTabData[6],
					["showTitle"] = dmgTabData[7], ["showEncounters"] = dmgTabData[8], ["showTargets"] = dmgTabData[9], ["showExtraButtons"] = dmgTabData[10],
					["expandedStatHeadings"] = {
						["totals"] = dmgTabData[11][1], ["crits"] = dmgTabData[11][2],
						["avoidance"] = dmgTabData[11][3], ["fullAvoids"] = dmgTabData[11][4], ["partials"] = dmgTabData[11][5],
						["other"] = dmgTabData[11][6], ["dmgTypes"] = dmgTabData[11][7]
					}
				},
				
				["takenTab"] = {
					["indexInWindow"] = takenTabData[1], ["selected"] = takenTabData[2], ["windowID"] = takenTabData[3], ["statsWindowID"] = takenTabData[4],
					["statsWindowVisibility"] = (takenTabData[5] == 1 and "Hide" or (takenTabData[5] == 2 and "Hover" or "Show")), ["autoSelectPlayer"] = takenTabData[6],
					["showTitle"] = takenTabData[7], ["showEncounters"] = takenTabData[8], ["showTargets"] = takenTabData[9], ["showExtraButtons"] = takenTabData[10],
					["expandedStatHeadings"] = {
						["totals"] = takenTabData[11][1], ["crits"] = takenTabData[11][2],
						["avoidance"] = takenTabData[11][3], ["fullAvoids"] = takenTabData[11][4], ["partials"] = takenTabData[11][5],
						["other"] = takenTabData[11][6], ["dmgTypes"] = takenTabData[11][7]
					}
				},
				
				["healTab"] = {
					["indexInWindow"] = healTabData[1], ["selected"] = healTabData[2], ["windowID"] = healTabData[3], ["statsWindowID"] = healTabData[4],
					["statsWindowVisibility"] = (healTabData[5] == 1 and "Hide" or (healTabData[5] == 2 and "Hover" or "Show")), ["autoSelectPlayer"] = healTabData[6],
					["showTitle"] = healTabData[7], ["showEncounters"] = healTabData[8], ["showTargets"] = healTabData[9], ["showExtraButtons"] = healTabData[10],
					["expandedStatHeadings"] = {
						["totals"] = healTabData[11][1], ["crits"] = healTabData[11][2], ["tempMorale"] = true -- new temp morale heading is expanded by default
					}
				},
				
				["powerTab"] = {
					["indexInWindow"] = powerTabData[1], ["selected"] = powerTabData[2], ["windowID"] = powerTabData[3], ["statsWindowID"] = powerTabData[4],
					["statsWindowVisibility"] = (powerTabData[5] == 1 and "Hide" or (powerTabData[5] == 2 and "Hover" or "Show")), ["autoSelectPlayer"] = powerTabData[6],
					["showTitle"] = powerTabData[7], ["showEncounters"] = powerTabData[8], ["showTargets"] = powerTabData[9], ["showExtraButtons"] = powerTabData[10],
					["expandedStatHeadings"] = {
						["totals"] = powerTabData[11][1], ["crits"] = powerTabData[11][2]
					}
				},
				
				-- new buff and debuff tabs (not in any windows by default)
				["debuffTab"] = {},
				["buffTab"] = {}
			}
			
			-- 2) update window state data
			settings.windowStates = {}
			for windowId,windowState in pairs(settings.statOverviewWindowStates) do
				settings.windowStates[windowId] = {
					["x"] = windowState[1], ["y"] = windowState[2], ["w"] = windowState[9], ["h"] = windowState[10], -- only the unminimized window width is stored now
					["showBackground"] = windowState[5],
					["showTitleBar"] = (windowState[6] == true or (windowState[6] ~= false and windowState[6] > 0)), -- new method used to save title bar status
					["autoHideTabs"] = windowState[7], ["minimized"] = windowState[8]
				};
			end
			settings.statOverviewWindowStates = nil;
			
			-- 3) update stat window state data
			settings.statsWindowStates = {}
			for windowId,windowState in pairs(settings.statOverviewStatsWindowStates) do
				settings.statsWindowStates[windowId] = {
					["x"] = windowState[1], ["y"] = windowState[2], ["w"] = windowState[3], ["h"] = windowState[4]
				};
			end
			settings.statOverviewStatsWindowStates = nil;
			
			-- show confirm dialog when reset is clicked by default
			settings.confirmOnReset = true;
			
			-- default file dialog settings
			settings["fileDialog"] = {}
		end
		
		-- now that all the default settings have been loaded, update the settings versionNo to match the current plugin versionNo
		settings.versionNo = versionNo;
	end
	
	-- 2) Restore settings
	
	-- restore all windows (and build a temporary dictionary of window ids -> windows)
	local maxId = 1;
	local windows = {}
	local noStatOverviewWindows = 0;
	-- create std stat overview windows
	for windowId,windowState in pairs(settings.windowStates) do
		windows[windowId] = StatOverviewWindow.RestoreState(windowId,windowState);
		maxId = math.max(windowId,maxId);
		noStatOverviewWindows = noStatOverviewWindows + 1;
	end
	-- if there is only one std stat overview window, disable its close button
	if (noStatOverviewWindows == 1) then windows[next(windows)].titleContent:EnableClose(false) end
		-- create stat overview stats windows
	for windowId,windowState in pairs(settings.statsWindowStates) do
		windows[windowId] = StatOverviewStatsWindow.RestoreState(windowId,windowState);
		maxId = math.max(windowId,maxId);
	end
	_G.idCounter = maxId + 1; -- ensure ids are never duplicated
	
	-- sort tabs by index order so that the order of tabs in a window is maintained (note that the position/order of tabs that are not in a window
	--  is irrelevant, and likewise for tabs of the same index in different windows)
	local tabStates = {}
	for tabKey,tabState in pairs(settings.tabStates) do
		table.insert(tabStates,{ ["tabKey"] = tabKey, ["state"] = tabState });
	end
	
	table.sort(tabStates,function(a,b)
		if (a["state"]["indexInWindow"] == nil) then
			if (b["state"]["indexInWindow"] ~= nil) then
				return false;
			else
				return (a["tabKey"] < b["tabKey"]);
			end
		elseif (b["state"]["indexInWindow"] == nil) then
			return true;
		else
			return (a["state"]["indexInWindow"] < b["state"]["indexInWindow"]);
		end
	end);
	
	for _,tabState in pairs(tabStates) do
		local statOverviewTab = statOverviewTabs[tabState["tabKey"]];
		-- add the stats panel to its relevant windows
		if (not statOverviewTab.isBuffTab) then
			windows[tabState["state"]["statsWindowID"]]:AddPanel(statOverviewTab.statsPanel);
		end
		-- add the tab to its relevant window
		if (tabState["state"]["windowID"] ~= nil) then windows[tabState["state"]["windowID"]]:AddTab(statOverviewTab,tabState["state"]["selected"],nil,true) end
		-- show stats windows if applicable (but retain previous window ordering)
		statOverviewTab:SetShowStats(
			(tabState["state"]["statsWindowVisibility"] and
				(tabState["state"]["statsWindowVisibility"] == "Hide" and 1 or (tabState["state"]["statsWindowVisibility"] == "Hover" and 2 or 3)) or
			nil),true);
		-- configure tab settings
		statOverviewTab.autoSelectPlayer = tabState["state"]["autoSelectPlayer"];
		statOverviewTab:SetShowTitle(tabState["state"]["showTitle"],true);
		statOverviewTab:SetShowEncounters(tabState["state"]["showEncounters"],true);
		statOverviewTab:SetShowTargets(tabState["state"]["showTargets"],true);
		statOverviewTab:SetShowSendChat(tabState["state"]["showExtraButtons"],true);
		if (statOverviewTab.window ~= nil) then statOverviewTab.window:Layout() end
		-- expand/collapse the stats tree views appropriately
		if (not statOverviewTab.isBuffTab) then
			statOverviewTab.statsPanel:Restore(tabState["state"]["expandedStatHeadings"]);
		end
	end
	
	-- now that tabs have been added to the relevant windows, set them to visible
	for windowId,_ in pairs(settings.windowStates) do
		local window = windows[windowId];
		
		if (#window.tabs > 0) then
			-- check to make sure that a tab is selected (should always be true)
			if (window.selected == nil) then
				window:SetSelectedTab(window.tabs[1],true,true);
			end
			windows[windowId]:SetVisible(true);
		end
	end
	
	-- set the global variables
	chatMenu:RestoreState(settings.chatChannel);
	Misc.SetValue("autoSelectNewEncounters",settings.autoSelectNewEncounters);
	Misc.SetValue("confirmOnReset",settings.confirmOnReset);
	
	-- set the file dialog settings
	fileSelectDialog.RestoreState(settings.fileDialog);
	
	-- now that settings have been restored, encode them all for quick saving later on
	_G.settings = EncodeNumbers(settings);
	-- if the settings file has had defaults added, store these values
	if (not upToDate) then SaveSettings() end
end

-- global function to encode numbers into a save-proof format (in place)
function _G.EncodeNumbers(object)
	if (type(object) == "number") then
		return ("<num>%f</num>"):format(object);
	elseif (type(object) ~= "table") then
		return object;
	else
		local newTable = {};
		for index, value in pairs(object) do
			newTable[EncodeNumbers(index)] = EncodeNumbers(value);
		end
		return newTable;
	end
end

-- global function to save current settings
--   (note that this function should be called anytime settings change -
--     The lib method automatically restricts saving to every 10s or so)
function _G.SaveSettings()
	Turbine.PluginData.Save(Turbine.DataScope.Character, "CombatAnalysis", settings);
end
